-- Compiled with roblox-ts v2.1.1
local TS = _G[script]
local exports = {}
--[[
	MatterBootstrapper.ts
	@author typechecked
	@date 5/12/2024
	A simple class that handles everything needed to bootstrap a Matter world including hot-reloading, live debug, networking, and component replication
]]
local Loader = TS.import(script, TS.getModule(script, "@rbxts", "loader").out).Loader
local _matter = TS.import(script, TS.getModule(script, "@rbxts", "matter").lib)
local Loop = _matter.Loop
local World = _matter.World
local RunService = TS.import(script, TS.getModule(script, "@rbxts", "services")).RunService
local Net = TS.import(script, TS.getModule(script, "@rbxts", "yetanothernet").lib)
local MatterHotReload = TS.import(script, script, "hotReload").default
local MatterDebugger = TS.import(script, script, "debugger").default
local MatterReplication = TS.import(script, script, "replication").default
for _k, _v in TS.import(script, script, "replicated") or {} do
	exports[_k] = _v
end
local MatterBootstrapper
do
	MatterBootstrapper = setmetatable({}, {
		__tostring = function()
			return "MatterBootstrapper"
		end,
	})
	MatterBootstrapper.__index = MatterBootstrapper
	function MatterBootstrapper.new(...)
		local self = setmetatable({}, MatterBootstrapper)
		return self:constructor(...) or self
	end
	function MatterBootstrapper:constructor(systemContainer, components, routes, enableReplication, enableHotReload, enableDebugger, debuggerKeycode)
		if routes == nil then
			routes = {}
		end
		if enableReplication == nil then
			enableReplication = true
		end
		if enableHotReload == nil then
			enableHotReload = RunService:IsStudio()
		end
		if enableDebugger == nil then
			enableDebugger = true
		end
		if debuggerKeycode == nil then
			debuggerKeycode = Enum.KeyCode.BackSlash
		end
		self.systemContainer = systemContainer
		self.components = components
		self.routes = routes
		self.world = World.new()
		self.loop = Loop.new()
		self.systems = Loader.Load(self.systemContainer)
		self.loop = Loop.new(self.world, self.widgets, self.routes)
		self.hotReload = MatterHotReload.new(self.loop, self.systemContainer)
		self.replication = MatterReplication.new(self.loop, self.routes, self.components)
		self.debugger = MatterDebugger.new(self.loop)
		self.widgets = self.debugger.debugger:getWidgets()
		if enableHotReload then
			self.hotReload:setupHotReload()
		else
			self.loop:scheduleSystems(self.systems)
		end
		local _ = enableDebugger and self.debugger:setupDebugger(debuggerKeycode)
		if enableReplication then
			self.replication:setupReplication()
		else
			Net.start(self.loop, self.routes)
		end
		self.loop:begin({
			default = RunService.Heartbeat,
			preRender = RunService.PreRender,
			preSimulation = RunService.PreSimulation,
			preAnimation = RunService.PreAnimation,
			renderStepped = RunService.RenderStepped,
			heartbeat = RunService.Heartbeat,
			postSimulation = RunService.PostSimulation,
			stepped = RunService.Stepped,
		})
	end
end
exports.default = MatterBootstrapper
return exports
